@@ -1,4 +1,5 @@
 """Beaker utilities"""
+
 try:
     import thread as _thread
     import threading as _threading
@@ -8,100 +9,70 @@ except ImportError:
 
 from datetime import datetime, timedelta
 import os
+import re
 import string
 import types
 import weakref
 import warnings
+import sys
+import inspect
 
-try:
-    Set = set
-except NameError:
-    from sets import Set
-try:
-    from hashlib import sha1
-except ImportError:
-    from sha import sha as sha1
+py3k = getattr(sys, 'py3kwarning', False) or sys.version_info >= (3, 0)
+py24 = sys.version_info < (2,5)
+jython = sys.platform.startswith('java')
+
+if py3k or jython:
+    import pickle
+else:
+    import cPickle as pickle
 
 from beaker.converters import asbool
-
-try:
-    from base64 import b64encode, b64decode
-except ImportError:
-    import binascii
-
-    _translation = [chr(_x) for _x in range(256)]
-
-    # From Python 2.5 base64.py
-    def _translate(s, altchars):
-        translation = _translation[:]
-        for k, v in altchars.items():
-            translation[ord(k)] = v
-        return s.translate(''.join(translation))
-
-    def b64encode(s, altchars=None):
-        """Encode a string using Base64.
-
-        s is the string to encode.  Optional altchars must be a string of at least
-        length 2 (additional characters are ignored) which specifies an
-        alternative alphabet for the '+' and '/' characters.  This allows an
-        application to e.g. generate url or filesystem safe Base64 strings.
-
-        The encoded string is returned.
-        """
-        # Strip off the trailing newline
-        encoded = binascii.b2a_base64(s)[:-1]
-        if altchars is not None:
-            return _translate(encoded, {'+': altchars[0], '/': altchars[1]})
-        return encoded
-
-    def b64decode(s, altchars=None):
-        """Decode a Base64 encoded string.
-
-        s is the string to decode.  Optional altchars must be a string of at least
-        length 2 (additional characters are ignored) which specifies the
-        alternative alphabet used instead of the '+' and '/' characters.
-
-        The decoded string is returned.  A TypeError is raised if s were
-        incorrectly padded or if there are non-alphabet characters present in the
-        string.
-        """
-        if altchars is not None:
-            s = _translate(s, {altchars[0]: '+', altchars[1]: '/'})
-        try:
-            return binascii.a2b_base64(s)
-        except binascii.Error, msg:
-            # Transform this exception for consistency
-            raise TypeError(msg)
-
-try:
-    from threading import local as _tlocal
-except ImportError:
-    try:
-        from dummy_threading import local as _tlocal
-    except ImportError:
-        class _tlocal(object):
-            def __init__(self):
-                self.__dict__['_tdict'] = {}
-
-            def __delattr__(self, key):
-                try:
-                    del self._tdict[(thread.get_ident(), key)]
-                except KeyError:
-                    raise AttributeError(key)
-
-            def __getattr__(self, key):
-                try:
-                    return self._tdict[(thread.get_ident(), key)]
-                except KeyError:
-                    raise AttributeError(key)
-
-            def __setattr__(self, key, value):
-                self._tdict[(thread.get_ident(), key)] = value
+from beaker import exceptions
+from threading import local as _tlocal
 
 
 __all__  = ["ThreadLocal", "Registry", "WeakValuedRegistry", "SyncDict",
             "encoded_path", "verify_directory"]
 
+def function_named(fn, name):
+    """Return a function with a given __name__.
+
+    Will assign to __name__ and return the original function if possible on
+    the Python implementation, otherwise a new function will be constructed.
+
+    """
+    fn.__name__ = name
+    return fn
+
+def skip_if(predicate, reason=None):
+    """Skip a test if predicate is true."""
+    reason = reason or predicate.__name__
+
+    from nose import SkipTest
+    def decorate(fn):
+        fn_name = fn.__name__
+        def maybe(*args, **kw):
+            if predicate():
+                msg = "'%s' skipped: %s" % (
+                    fn_name, reason)
+                raise SkipTest(msg)
+            else:
+                return fn(*args, **kw)
+        return function_named(maybe, fn_name)
+    return decorate
+
+def assert_raises(except_cls, callable_, *args, **kw):
+    """Assert the given exception is raised by the given function + arguments."""
+
+    try:
+        callable_(*args, **kw)
+        success = False
+    except except_cls, e:
+        success = True
+ 
+    # assert outside the block so it works for AssertionError too !
+    assert success, "Callable did not raise an exception"
+
 
 def verify_directory(dir):
     """verifies and creates a directory.  tries to
@@ -116,53 +87,75 @@ def verify_directory(dir):
             if tries > 5:
                 raise
 
-    
-def deprecated(func, message):
-    def deprecated_method(*args, **kargs):
-        warnings.warn(message, DeprecationWarning, 2)
-        return func(*args, **kargs)
-    try:
-        deprecated_method.__name__ = func.__name__
-    except TypeError: # Python < 2.4
-        pass
-    deprecated_method.__doc__ = "%s\n\n%s" % (message, func.__doc__)
-    return deprecated_method
+def has_self_arg(func):
+    """Return True if the given function has a 'self' argument."""
+    args = inspect.getargspec(func)
+    if args and args[0] and args[0][0] in ('self', 'cls'):
+        return True
+    else:
+        return False
 
-class ThreadLocal(_tlocal):
+
+def warn(msg, stacklevel=3):
+    """Issue a warning."""
+    if isinstance(msg, basestring):
+        warnings.warn(msg, exceptions.BeakerWarning, stacklevel=stacklevel)
+    else:
+        warnings.warn(msg, stacklevel=stacklevel)
+
+
+def deprecated(message):
+    def wrapper(fn):
+        def deprecated_method(*args, **kargs):
+            warnings.warn(message, DeprecationWarning, 2)
+            return fn(*args, **kargs)
+        # TODO: use decorator ?  functools.wrapper ?
+        deprecated_method.__name__ = fn.__name__
+        deprecated_method.__doc__ = "%s\n\n%s" % (message, fn.__doc__)
+        return deprecated_method
+    return wrapper
+
+
+class ThreadLocal(object):
     """stores a value on a per-thread basis"""
-    
+
+    __slots__ = '_tlocal'
+
+    def __init__(self):
+        self._tlocal = _tlocal()
+
     def put(self, value):
-        self.value = value
-    
+        self._tlocal.value = value
+
     def has(self):
-        return hasattr(self, 'value')
-            
+        return hasattr(self._tlocal, 'value')
+
     def get(self, default=None):
-        return getattr(self, 'value', default)
-            
+        return getattr(self._tlocal, 'value', default)
+
     def remove(self):
-        del self.value
-    
+        del self._tlocal.value
+
 class SyncDict(object):
     """
     An efficient/threadsafe singleton map algorithm, a.k.a.
     "get a value based on this key, and create if not found or not
     valid" paradigm:
-    
+
         exists && isvalid ? get : create
 
-    Works with weakref dictionaries and the LRUCache to handle items
-    asynchronously disappearing from the dictionary.  
+    Designed to work with weakref dictionaries to expect items
+    to asynchronously disappear from the dictionary.
 
     Use python 2.3.3 or greater !  a major bug was just fixed in Nov.
     2003 that was driving me nuts with garbage collection/weakrefs in
     this section.
 
-    """    
+    """
     def __init__(self):
         self.mutex = _thread.allocate_lock()
         self.dict = {}
-        
+
     def get(self, key, createfunc, *args, **kwargs):
         try:
             if self.has_key(key):
@@ -191,7 +184,7 @@ class SyncDict(object):
 
     def has_key(self, key):
         return self.dict.has_key(key)
-        
+
     def __contains__(self, key):
         return self.dict.__contains__(key)
     def __getitem__(self, key):
@@ -209,28 +202,45 @@ class WeakValuedRegistry(SyncDict):
         self.mutex = _threading.RLock()
         self.dict = weakref.WeakValueDictionary()
 
-            
+sha1 = None
 def encoded_path(root, identifiers, extension = ".enc", depth = 3,
                  digest_filenames=True):
+
     """Generate a unique file-accessible path from the given list of
     identifiers starting at the given root directory."""
-    ident = string.join(identifiers, "_")
+    ident = "_".join(identifiers)
+
+    global sha1
+    if sha1 is None:
+        from beaker.crypto import sha1
 
     if digest_filenames:
-        ident = sha1(ident).hexdigest()
-    
+        if py3k:
+            ident = sha1(ident.encode('utf-8')).hexdigest()
+        else:
+            ident = sha1(ident).hexdigest()
+
     ident = os.path.basename(ident)
 
     tokens = []
     for d in range(1, depth):
         tokens.append(ident[0:d])
-    
+
     dir = os.path.join(root, *tokens)
     verify_directory(dir)
-    
+
     return os.path.join(dir, ident + extension)
 
 
+def asint(obj):
+    if isinstance(obj, int):
+        return obj
+    elif isinstance(obj, basestring) and re.match(r'^\d+$', obj):
+        return int(obj)
+    else:
+        raise Exception("This is not a proper int")
+
+
 def verify_options(opt, types, error):
     if not isinstance(opt, types):
         if not isinstance(types, tuple):
@@ -243,6 +253,11 @@ def verify_options(opt, types, error):
                 else:
                     if typ == bool:
                         typ = asbool
+                    elif typ == int:
+                        typ = asint
+                    elif typ in (timedelta, datetime):
+                        if not isinstance(opt, typ):
+                            raise Exception("%s requires a timedelta type", typ)
                     opt = typ(opt)
                 coerced = True
             except:
@@ -251,6 +266,8 @@ def verify_options(opt, types, error):
                 break
         if not coerced:
             raise Exception(error)
+    elif isinstance(opt, str) and not opt.strip():
+        raise Exception("Empty strings are invalid for: %s" % error)
     return opt
 
 
@@ -265,11 +282,11 @@ def coerce_session_params(params):
     rules = [
         ('data_dir', (str, types.NoneType), "data_dir must be a string "
          "referring to a directory."),
-        ('lock_dir', (str,), "lock_dir must be a string referring to a "
+        ('lock_dir', (str, types.NoneType), "lock_dir must be a string referring to a "
          "directory."),
         ('type', (str, types.NoneType), "Session type must be a string."),
-        ('cookie_expires', (bool, datetime, timedelta), "Cookie expires was "
-         "not a boolean, datetime, or timedelta instance."),
+        ('cookie_expires', (bool, datetime, timedelta, int), "Cookie expires was "
+         "not a boolean, datetime, int, or timedelta instance."),
         ('cookie_domain', (str, types.NoneType), "Cookie domain must be a "
          "string."),
         ('id', (str,), "Session id must be a string."),
@@ -280,18 +297,26 @@ def coerce_session_params(params):
         ('encrypt_key', (str, types.NoneType), "Session validate_key must be "
          "a string."),
         ('secure', (bool, types.NoneType), "Session secure must be a boolean."),
+        ('httponly', (bool, types.NoneType), "Session httponly must be a boolean."),
         ('timeout', (int, types.NoneType), "Session timeout must be an "
          "integer."),
         ('auto', (bool, types.NoneType), "Session is created if accessed."),
+        ('webtest_varname', (str, types.NoneType), "Session varname must be "
+         "a string."),
     ]
-    return verify_rules(params, rules)
+    opts = verify_rules(params, rules)
+    cookie_expires = opts.get('cookie_expires')
+    if cookie_expires and isinstance(cookie_expires, int) and \
+       not isinstance(cookie_expires, bool):
+        opts['cookie_expires'] = timedelta(seconds=cookie_expires)
+    return opts
 
 
 def coerce_cache_params(params):
     rules = [
         ('data_dir', (str, types.NoneType), "data_dir must be a string "
          "referring to a directory."),
-        ('lock_dir', (str,), "lock_dir must be a string referring to a "
+        ('lock_dir', (str, types.NoneType), "lock_dir must be a string referring to a "
          "directory."),
         ('type', (str,), "Cache type must be a string."),
         ('enabled', (bool, types.NoneType), "enabled must be true/false "
@@ -299,7 +324,9 @@ def coerce_cache_params(params):
         ('expire', (int, types.NoneType), "expire must be an integer representing "
          "how many seconds the cache is valid for"),
         ('regions', (list, tuple, types.NoneType), "Regions must be a "
-         "comma seperated list of valid regions")
+         "comma seperated list of valid regions"),
+        ('key_length', (int, types.NoneType), "key_length must be an integer "
+         "which indicates the longest a key can be before hashing"),
     ]
     return verify_rules(params, rules)
 
@@ -307,6 +334,7 @@ def coerce_cache_params(params):
 def parse_cache_config_options(config, include_defaults=True):
     """Parse configuration options and validate for use with the
     CacheManager"""
+
     # Load default cache options
     if include_defaults:
         options= dict(type='memory', data_dir=None, expire=None, 
@@ -319,25 +347,29 @@ def parse_cache_config_options(config, include_defaults=True):
         if key.startswith('cache.'):
             options[key[6:]] = val
     coerce_cache_params(options)
-    
+
     # Set cache to enabled if not turned off
-    if 'enabled' not in options:
+    if 'enabled' not in options and include_defaults:
         options['enabled'] = True
-    
+
     # Configure region dict if regions are available
     regions = options.pop('regions', None)
     if regions:
         region_configs = {}
         for region in regions:
+            if not region: # ensure region name is valid
+                continue
             # Setup the default cache options
             region_options = dict(data_dir=options.get('data_dir'),
                                   lock_dir=options.get('lock_dir'),
                                   type=options.get('type'),
                                   enabled=options['enabled'],
-                                  expire=options.get('expire'))
-            region_len = len(region) + 1
+                                  expire=options.get('expire'),
+                                  key_length=options.get('key_length', 250))
+            region_prefix = '%s.' % region
+            region_len = len(region_prefix)
             for key in options.keys():
-                if key.startswith('%s.' % region):
+                if key.startswith(region_prefix):
                     region_options[key[region_len:]] = options.pop(key)
             coerce_cache_params(region_options)
             region_configs[region] = region_options
@@ -350,8 +382,8 @@ def func_namespace(func):
     if hasattr(func, 'im_func'):
         kls = func.im_class
         func = func.im_func
-    
+
     if kls:
         return '%s.%s' % (kls.__module__, kls.__name__)
     else:
-        return func.__module__
+        return '%s|%s' % (inspect.getsourcefile(func), func.__name__)
