@@ -1,17 +1,34 @@
 # coding: utf-8
+import mock
 import os
 
-from beaker.cache import clsmap, Cache
+from beaker.cache import clsmap, Cache, util
 from beaker.middleware import CacheMiddleware, SessionMiddleware
+from beaker.exceptions import InvalidCacheBackendError
 from nose import SkipTest
-from webtest import TestApp
+import unittest
 
-if isinstance(clsmap.get('ext:memcached'), Exception):
-    raise SkipTest("'memcache' or 'cmemcache' is not installed, can't test "
-                   "memcached backend")
+try:
+    from webtest import TestApp
+except ImportError:
+    TestApp = None
+
+try:
+    from beaker.ext import memcached
+    client = memcached._load_client()
+except InvalidCacheBackendError:
+    raise SkipTest("an appropriate memcached backend is not installed")
 
 mc_url = '127.0.0.1:11211'
-loc = os.path.sep.join([os.path.dirname(os.path.abspath(__file__)), 'sessions'])
+
+c =client.Client([mc_url])
+c.set('x', 'y')
+if not c.get('x'):
+    raise SkipTest("Memcached is not running at %s" % mc_url)
+
+def teardown():
+    import shutil
+    shutil.rmtree('./cache', True)
 
 def simple_session_app(environ, start_response):
     session = environ['beaker.session']
@@ -91,7 +108,7 @@ def cache_manager_app(environ, start_response):
         yield "test_key wasn't cleared, is: %s\n" % \
             cm.get_cache('test')['test_key']
 
-
+@util.skip_if(lambda: TestApp is None, "webtest not installed")
 def test_session():
     app = TestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
     res = app.get('/')
@@ -102,6 +119,7 @@ def test_session():
     assert 'current value is: 3' in res
 
 
+@util.skip_if(lambda: TestApp is None, "webtest not installed")
 def test_session_invalid():
     app = TestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
     res = app.get('/invalid', headers=dict(Cookie='beaker.session.id=df7324911e246b70b5781c3c58328442; Path=/'))
@@ -126,31 +144,31 @@ def test_dropping_keys():
     assert cache.has_key('test')
     assert 'test' in cache
     assert cache.has_key('fred')
-    
+
     # Directly nuke the actual key, to simulate it being removed by memcached
     cache.namespace.mc.delete('test_test')
     assert not cache.has_key('test')
     assert cache.has_key('fred')
-    
+
     # Nuke the keys dict, it might die, who knows
     cache.namespace.mc.delete('test:keys')
     assert cache.has_key('fred')
-    
+
     # And we still need clear to work, even if it won't work well
     cache.clear()
 
 def test_deleting_keys():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     cache.set_value('test', 20)
-    
+
     # Nuke the keys dict, it might die, who knows
     cache.namespace.mc.delete('test:keys')
-    
+
     assert cache.has_key('test')
-    
+
     # make sure we can still delete keys even though our keys dict got nuked
     del cache['test']
-    
+
     assert not cache.has_key('test')
 
 def test_has_key_multicache():
@@ -162,7 +180,7 @@ def test_has_key_multicache():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     assert cache.has_key("test")
 
-def test_unicode_keys():    
+def test_unicode_keys():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     o = object()
     cache.set_value(u'hiŏ', o)
@@ -171,7 +189,7 @@ def test_unicode_keys():
     cache.remove_value(u'hiŏ')
     assert u'hiŏ' not in cache
 
-def test_spaces_in_unicode_keys():    
+def test_spaces_in_unicode_keys():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     o = object()
     cache.set_value(u'hi ŏ', o)
@@ -189,6 +207,7 @@ def test_spaces_in_keys():
     assert cache.has_key("hasspace")
     assert 42 == cache.get_value("hasspace")
 
+@util.skip_if(lambda: TestApp is None, "webtest not installed")
 def test_increment():
     app = TestApp(CacheMiddleware(simple_app))
     res = app.get('/', extra_environ={'beaker.clear':True})
@@ -197,7 +216,7 @@ def test_increment():
     assert 'current value is: 2' in res
     res = app.get('/')
     assert 'current value is: 3' in res
-    
+
     app = TestApp(CacheMiddleware(simple_app))
     res = app.get('/', extra_environ={'beaker.clear':True})
     assert 'current value is: 1' in res
@@ -206,15 +225,65 @@ def test_increment():
     res = app.get('/')
     assert 'current value is: 3' in res
 
+@util.skip_if(lambda: TestApp is None, "webtest not installed")
 def test_cache_manager():
     app = TestApp(CacheMiddleware(cache_manager_app))
     res = app.get('/')
     assert 'test_key is: test value' in res
     assert 'test_key cleared' in res
 
+@util.skip_if(lambda: TestApp is None, "webtest not installed")
 def test_store_none():
     app = TestApp(CacheMiddleware(using_none_app))
     res = app.get('/', extra_environ={'beaker.clear':True})
     assert 'current value is: 10' in res
     res = app.get('/')
     assert 'current value is: None' in res
+
+class TestPylibmcInit(unittest.TestCase):
+    def setUp(self):
+
+        from beaker.ext import memcached
+        try:
+            import pylibmc as memcache
+        except:
+            import memcache
+            memcached._client_libs['pylibmc'] = memcached.pylibmc = memcache
+            from contextlib import contextmanager
+            class ThreadMappedPool(dict):
+                "a mock of pylibmc's ThreadMappedPool"
+
+                def __init__(self, master):
+                    self.master = master
+
+                @contextmanager
+                def reserve(self):
+                    yield self.master
+            memcache.ThreadMappedPool = ThreadMappedPool
+
+    def test_uses_pylibmc_client(self):
+        from beaker.ext import memcached
+        cache = Cache('test', data_dir='./cache', 
+                            memcache_module='pylibmc', 
+                            url=mc_url, type="ext:memcached")
+        assert isinstance(cache.namespace, memcached.PyLibMCNamespaceManager)
+
+    def test_dont_use_pylibmc_client(self):
+        from beaker.ext.memcached import _load_client
+        load_mock = mock.Mock()
+        load_mock.return_value = _load_client('memcache')
+        with mock.patch('beaker.ext.memcached._load_client', load_mock):
+            cache = Cache('test', data_dir='./cache', url=mc_url, type="ext:memcached")
+            assert not isinstance(cache.namespace, memcached.PyLibMCNamespaceManager)
+            assert isinstance(cache.namespace, memcached.MemcachedNamespaceManager)
+
+    def test_client(self):
+        cache = Cache('test', data_dir='./cache', url=mc_url, type="ext:memcached")
+        o = object()
+        cache.set_value("test", o)
+        assert cache.has_key("test")
+        assert "test" in cache
+        assert not cache.has_key("foo")
+        assert "foo" not in cache
+        cache.remove_value("test")
+        assert not cache.has_key("test")
